module chem_param

  use binas, only : xmair

  implicit none

  ! define specific data structures 

  ! definition of the chemistry: #reactions, order of species, etc.
  ! parameters needed for chemistry and rate constants 
  !

  ! nmark: number of 'marked' tracers 
  integer, parameter :: nmark = 1
  character(len=8),dimension(nmark) :: marknam = (/ 'ocean   ' /)

  ! ntrace: number of tracers for chemistry
  integer, parameter :: ntrace  = 150
  ! ntracet: number of transported tracers
  integer, parameter :: ntracet = 150
  ! ntrace_chem: non-transported tracers   
  integer, parameter :: ntrace_chem  = ntrace-ntracet
  ! maxtrace: total number of tracers
  integer, parameter :: maxtrace = ntrace + 0 

  !
  ! components numbers
  !
  integer, parameter :: ico2m=1  ! mean co2 mixing ratio
  !
  ! additional fields used in chemistry routine alone 
  ! (more meteo-like files in units different from #/cm3)
  !
  integer,parameter                   :: n_extra = 0
  integer,parameter                   :: nstd    = 1
  integer, dimension(nstd), parameter :: istd    = (/ 1 /)
  !
  ! species name
  !
  character(len=8), dimension(maxtrace),parameter :: names = 'co2m'
  ! 
  ! some stuff for the budgets....
  !
  integer, parameter           :: njnum = 1
  integer, parameter           :: nreac = 1
  integer, parameter           :: nhet = 1
  integer, parameter           :: nreacw = 1
  character(len=8), dimension(njnum)  :: jnam   = (/'dummy   '/)
  character(len=8), dimension(nreac)  :: ratnam = (/'dummy   '/)
  character(len=8), dimension(nreacw) :: rwnam  = (/'dummy   '/)
  character(len=8), dimension(nhet) :: rhetnam  = (/'dummy   '/)

  ! molar weights of components

  real,    parameter :: xmh=1.0079
  real,    parameter :: xmn=14.0067
  real,    parameter :: xmc=12.01115
  real,    parameter :: xms=32.064
  real,    parameter :: xmo=15.9994
  real,    parameter :: xmcl=35.453
  real,    parameter :: xmf=18.9984032
  real,    parameter :: xmh2o=xmh*2+xmo
  real,    parameter :: xmco2=xmc+2*xmo
  real,    parameter :: xmrn222=222.
  real,    parameter :: xmpb210=210.
  real,    parameter :: xmsf6= xms+6*xmf
  real,    parameter :: xmnmv=999. ! this is a dummy molecular mass

  real,dimension(maxtrace),parameter      :: ra=xmco2
  real,dimension(maxtrace),parameter      :: fscale = xmair/ra
  real,dimension(maxtrace),parameter      :: uscale = 1.e6

  !   fscale(ntrace): scaling factor for conversion of mixing ratios 
  !   in kg tracer per kg air to practical mixing ratio units (e.g. ppm)
  !   In this version: ratio of molecular weight  of tracer to that of air 

end module chem_param
