module emission_co2_bio
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform co2 biosphere emissions 
  !
  !       interface
  !       ---------
  !       call emission_apply_bio
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_emission
  !-----------------------------------------------------------------------
  use Dims,  only         : nregions, nlon360,nlat180
  use Meteo, only         : t2m_sfc,ssr_sfc
  use Meteo, only         : Set
  implicit none

  private

  public   :: co2_bio
  ! public routines:
  public   :: declare_emission_co2_bio, free_emission_co2_bio, calc_emission_co2_bio

  real,dimension(nlon360,nlat180),target :: co2_bio
  logical                                :: use_fires


contains

  subroutine declare_emission_co2_bio
  use GO,                  only : TrcFile, Init, Done, ReadRc
  use global_data,         only : rcfile

  implicit none
  integer                                      :: status
  type(TrcFile)        :: rcF

   call Init( rcF, rcfile, status )
   call ReadRc( rcF, 'emis.use.fires', use_fires ,status ,default=.false.)
   call Done( rcF, status )

  call Set(   t2m_sfc, status, used=.true. )
  call Set(   ssr_sfc, status, used=.true. )

  end subroutine declare_emission_co2_bio

  subroutine calc_emission_co2_bio
  use dims,        only: nlon360, nlat180, im,jm, sec_month, newsrun, idate, itau, staggered, nread, newmonth
  use io_save,     only: readtm3hdf
  use toolbox,     only: escape_tm
  use emission_data,	only: chardate
  use Meteo      , only : Set

  implicit none
  integer                                      :: status
  integer                                      :: region, imr, jmr
  integer, parameter                           :: annual_field = 0, rank2 = 2, level1 = 1
  character(len=10)                            :: cdate
  real,dimension(nlon360,nlat180)              :: p0,r0
  real,pointer,dimension(:,:)                  :: t2m,ssr

  if(mod(itau,nread) /= 0) return  ! only every nread hours

  if(newmonth) then
      call chardate(cdate,idate,6)
      if(use_fires) then
          call readtm3hdf('biofireparams_'//trim(cdate)//'.hdf',rank2,nlon360,nlat180,level1,annual_field,r0,'rparam1')
          call readtm3hdf('biofireparams_'//trim(cdate)//'.hdf',rank2,nlon360,nlat180,level1,annual_field,p0,'pparam1')
      else 
          call readtm3hdf('bioparams_'//trim(cdate)//'.hdf',rank2,nlon360,nlat180,level1,annual_field,r0,'rparam1')
          call readtm3hdf('bioparams_'//trim(cdate)//'.hdf',rank2,nlon360,nlat180,level1,annual_field,p0,'pparam1')
      endif
  endif

  t2m=> t2m_sfc%data(:,:,1)
  ssr=> ssr_sfc%data(:,:,1)
   
  co2_bio=r0*(1.5**((t2m-273.15)/10.0))  ! Q10 relation for repsiration
  co2_bio=co2_bio+p0*ssr        ! radiation scaling for GPP 

  nullify(t2m)
  nullify(ssr)

  end subroutine calc_emission_co2_bio

   subroutine free_emission_co2_bio
    implicit none
    integer               :: region
   end subroutine free_emission_co2_bio

end module emission_co2_bio
