module emission_co2_fires
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform co2 fire emissions 
  !
  !       interface
  !       ---------
  !       call emission_apply_fires
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_emission
  !-----------------------------------------------------------------------
  use Dims,          only: nregions, nlon360,nlat180
  use emission_data, only: gridbox_per_month_to_flux
  use chem_param ,   only: xmco2, xmc
  implicit none

  private

  public   :: co2_fires
  ! public routines:
  public   :: declare_emission_co2_fires, free_emission_co2_fires, calc_emission_co2_fires

  real,dimension(nlon360,nlat180) :: co2_fires
  logical                         :: use_fires


contains
  subroutine declare_emission_co2_fires
  use GO,                  only : TrcFile, Init, Done, ReadRc
  use global_data,         only : rcfile

  implicit none
  type(TrcFile)        :: rcF
  integer              :: status

  ! nothing to be done
   call Init( rcF, rcfile, status )
   call ReadRc( rcF, 'emis.use.fires', use_fires ,status ,default=.false.)
   call Done( rcF, status )

  end subroutine declare_emission_co2_fires

  subroutine calc_emission_co2_fires
  use dims,        only: nlon360, nlat180, im,jm, sec_month, newsrun, idate, newmonth, sec_year
  use io_save,     only: readtm3hdf
  use emission_data,    only: chardate
  implicit none
  integer                                      :: status
  integer                                      :: region, imr, jmr
  integer, parameter                           :: annual_field = 0, rank2 = 2, level1 = 1
  character(len=10)                            :: cdate
  real                                         :: year2month

  if(newmonth.or.newsrun) then
     
     if(use_fires) then
      call chardate(cdate,idate,6)
      call readtm3hdf('BBguido1x1.hdf'//trim(cdate),rank2,nlon360,nlat180,&
                         level1,idate(2)-1,co2_fires,'BB_co2')
      co2_fires=co2_fires/xmc ! to mol/m2/s
     else
      co2_fires=0.0
     endif
  endif
   
  end subroutine calc_emission_co2_fires

   subroutine free_emission_co2_fires
    implicit none
    integer               :: region
   end subroutine free_emission_co2_fires

end module emission_co2_fires
