!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module initexit

  use GO, only : gol, goErr, goPr
  
  implicit none

  ! --- in/out ------------------------------

  private

  public :: default, exitus, start


  ! --- const --------------------------------
    
  character(len=*), parameter  ::  mname = 'initexit'

  
contains


  subroutine default
    !---------------------------------------------------------------------
    ! set default values of control variables
    !
    !                             mh, 27-jun-1989
    !                             mh, 26-sep-1992
    !                             v 9.1
    !                              mk, 21-dec-2002
    !---------------------------------------------------------------------
    use dims
    use datetime,    only : tau2date
    use toolbox,     only : escape_tm
    implicit none

    ! local
    integer :: i,k,n

    ! start
    !                 ! set default calendar 
    !                 ! (real calendar, iyear0=1980)
    icalendo=2

    !                 ! default time steps of basic tasks
    nstep=0
    ndyn=3600*3
    nconv=3600*3
    ndiff=24*3600*1000 !never happes in one month
    ntrans=0
    ndiag=4*3600
    nchem=0
    nsrce=24*3600
    !nread=3*3600       ! <--- set in main program by Meteo_Init
    nwrite=-1
    ninst=0
    !c                       ! default is restart
    istart=10
    itaui=0
    newsrun=.true.
    call tau2date(itaui,idatei)
    if ( mod(idatei(4),3) /= 0 ) &
         call escape_tm('default: GMT start time should be multiple of 3')    
    itaue=itaui
    call tau2date(itaue,idatee)
    itaut=0
    call tau2date(itaut,idatet)


    !c                       ! output for conservation diagnostics
    !c                       ! -1: daily, -2: monthly, -3: yearly,
    !c                               ! >=0 interval in sec
    ndiagp1=-2

    !c                       ! output for mean field diagnostics
    !c                       ! -1: daily, -2: monthly, -3: yearly,
    !c                               ! >=0 interval in sec
    ndiagp2=-2

    !c                               ! full convection
    czeta=1.
    !c                               ! full vertical diffusion
    czetak=1.
    !c                               ! scaling factor for horizontal diffusion
    limits=.true.

    !c                       ! checking interval
    ncheck=6

    !c                       ! control for debug output
    cdebug=.false.
    okdebug=.false.
    revert = 1

  end subroutine default



  !--------------------------------------------------------------
  ! terminate a model run
  !
  ! icode=0   normal termination
  !       else      abnormal termination
  !
  ! msg       character string to be printed on unit kmain
  !
  !                             mh, 29-jun-1989
  !
  ! changed to v 9.1                        mh, 26-sep-1992
  !
  !--------------------------------------------------------------

  subroutine exitus

    use GO           , only : gol, goErr, goPr
    use GO           , only : goGetFU
    use dims         , only : nregions
    use dims         , only : nstep, itau
    use dims         , only : cpu0
    use dims         , only : nxi, xi, nloop_max
    use dims         , only : okdebug, kmain
    use global_data  , only : free_fields
    use io_save,       only : savehdf,savebinary
    use user_output,   only : user_output_done
    use sources_sinks, only : trace_end
    use datetime,      only : tstamp
    use ParTools     , only : myid, root, Par_Barrier, Par_StopMPI
    use tracer_data  , only : Par_Check_Domain
    use global_data  , only : outdir
    use advectm_cfl,    only : done_cfl
    use toolbox,     only : escape_tm

    implicit none

    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/exitus'
    
    ! --- local ------------------------------
    
    integer   ::  status
    integer   ::  fu
    integer   ::  region, gatheri
    real      ::  cpu3, gatherr

    ! --- begin ---------------------------------------

    do region=1,nregions      
       ! transfer data to tracer domain
       call Par_Check_Domain( region, 'n', 'tracer' ) 
    end do

    ! save the model state
    call SaveHDF( 'successful completion of run','save.hdf', status )
    if (status/=0) then; write (gol,'("in ",a,"; continue")') rname; call goErr; call escape_tm('stop'); end if

    ! complete user-specified output   !CMK only once!
    call user_output_done( status )
    if (status/=0) then; write (gol,'("in ",a,"; continue")') rname; call goErr; end if

    ! finish up chemistry/print budget, output shored-lived compounds
    call trace_end( 'successful completion of run',trim(outdir)//'/save', status )
    if (status/=0) then; write (gol,'("in ",a,"; continue")') rname; call goErr; end if

    call Par_Barrier
    if ( okdebug ) then
      write (gol,'("exitus: starting to free fields ",i4)') myid; call goPr
    end if

    call free_fields     !free memory of this run 

    call Par_Barrier
    if ( okdebug ) then
      write (gol,'("exitus: starting time computation",i4)') myid; call goPr
    end if
    call Par_Barrier

    if ( myid == root ) then
      write (gol,'(" ")'); call goPr
      write (gol,'("exitus: CFL info:")'); call goPr
      do region=1, nregions
        write (gol,'(a,3i4,f10.4)') ' exitus: x: region, nxi, nloop_max, xi',  &
               region, nxi(region,1), nloop_max(region,1), xi(region,1); call goPr
        write (gol,'(a,3i4,f10.4)') ' exitus: y: region, nxi, nloop_max, xi',  &
               region, nxi(region,2), nloop_max(region,2), xi(region,2); call goPr
        write (gol,'(a,3i4,f10.4)') ' exitus: z: region, nxi, nloop_max, xi',  &
               region, nxi(region,3), nloop_max(region,3), xi(region,3); call goPr
      end do
    end if
    
    ! cfl finished ...
    call Done_CFL

    call Par_Barrier

    if ( myid == root ) then

      write (gol,'(1x)'); call goPr
      write (gol,'(1x)'); call goPr
      write (gol,'(1x,39("--"))'); call goPr
      write (gol,'(1x,39("--"))'); call goPr
      write (gol,'(1x)'); call goPr
      write (gol,'("exitus: program has terminated normally.")'); call goPr

      call tstamp(kmain,itau,'exitus: final time')

      call cputim(cpu3)
      cpu3 = cpu3-cpu0

      write (gol,'(a,i8,a,f16.2,a,f16.8)') &
           ' exitus: no of timesteps: ',nstep,' cpu (s):',cpu3, &
           ' cpu/step: ',cpu3/nstep; call goPr

      write (gol,'(1x)'); call goPr
      write (gol,'(1x,39("--"))'); call goPr
      write (gol,'(1x,39("--"))'); call goPr
      write (gol,'(1x)'); call goPr

      ! write ok file:
      call goGetFU( fu, status )
      IF_NOTOK_RETURN(status=1)
      open( unit=fu, file='tm5.ok', form='formatted', status='unknown', iostat=status )
      if ( status/=0 ) then
        write (gol,'("from opening okfile")'); call goErr
      else
        write (fu,'("Program terminated normally")',iostat=status)
        if ( status/=0 ) then
          write (gol,'("from writing to okfile")'); call goErr
        else
          close( fu, iostat=status )
          if ( status/=0 ) then
            write (gol,'("from closing okfile")'); call goErr
          end if
        end if
      end if

    end if ! all Pe's from here 

    call Par_barrier
    if ( okdebug ) then
      write (gol,'("exitus: calling stopmpi ",i4)') myid; call goErr
    end if
    call Par_barrier
    call Par_StopMPI 

  end subroutine exitus
      
      

  !-----------------------------------------------------------------
  !     this subroutine performs the initialization of a model run or 
  !     its continuation.
  !
  !
  !     structure of control input :
  !     line 1 :     blank or name of alternative control input file
  !     line 2-3:    text for run header/info (2*80 char)
  !     line 4-... : namelist inputz
  !                  (must be terminated by string
  !                  '&end' in column 2-5)
  !     subsequent input may be read in sr trace0 (via unit kinput0)
  !
  !                                       mh,  1-dec-1990
  !
  !      changed to v 8.5                 mh, mpi HH     23-feb-1994
  !      changed to zoom version          MK
  !-----------------------------------------------------------------------

  subroutine start( reset, status )

    use GO           , only : gol, goPr, goErr, goLabel
    use GO           , only : TrcFile, Init, Done, ReadRc
    use go           , only : TDate, NewDate, IncrDate, operator(+), operator(-), rTotal
    use dims         , only : im, jm, dxy11, nlat180, nlon360
    use dims         , only : nregions, region_name
    use dims         , only : ndyn, nchem, nsrce, nconv, nread, ndyn_max
    use dims         , only : ndiag, ndiagp1, ndiagp2, nwrite, ninst, ndiff
    use dims         , only : nstep0, nstep
    use dims         , only : idatei, idatee, idatet, idate, revert
    use dims         , only : itaui, itaue, itaut, itau
    use dims         , only : iyear0, julday0, icalendo
    use dims         , only : newyr, newday, newmonth, newsrun
    use dims         , only : okdebug, cpu0, kmain
    use dims         , only : istart
    use dims         , only : adv_scheme
    use dims         , only : czeta, czetak, limits, cdebug
    use global_data,   only : rcfile, rcfile_runtime
    use global_data,   only : inputdir, outdir
    use global_data,   only : region_dat, mass_dat
    use global_data,   only : declare_fields, use_convec
    use Meteo        , only : Meteo_Setup_Mass, Meteo_Setup_Other
    use sources_sinks, only : trace0, trace1, start_co2
    use emission_co2_all, only : co2_avg,n_accumulate
    use redgridZoom  , only : grid_reduced, InitRedGrid
    use redgridZoom  , only : nred, nredmax, clustsize, imred, imredj, jred
    use geometry,      only : geomtryh, calc_dxy
    use io_save,       only : readhdf, readhdfmmr, savehdf, readhdfmmix, readbinary
    use user_output,   only : user_output_init
    use datetime,      only : date2tau, tau2date, julday, tstamp
    use zoom_tools,    only : coarsen_region, update_parent
    use advect,        only : dynam0
    use advect_tools,  only : advect_m, m2phlb, m2phlb1, coarsen_ambm
    use toolbox,     only : escape_tm
#ifdef MPI    
    use mpi_const, only : MPI_COMM_WORLD
    use mpi_const, only : MPI_INTEGER, MPI_LOGICAL, MPI_CHARACTER
    use mpi_const, only : MY_REAL
    use mpi_const, only : ierr
#endif
    use ParTools, only : myid, root, Par_Barrier
    use advectm_cfl  , only : Init_CFL, Check_CFL
    use user_input   , only : user_input_start

    ! --- in/out --------------------------------------------
    
    logical, intent(in)         :: reset
    integer, intent(out)        ::  status

    ! --- const ----------------------------------------

    character(len=*), parameter ::  rname = mname//'/start'
    
    ! --- local ---------------------------------------------
    
    integer             ::  n, region
    character(len=256)  ::  fname, fdir
    type(TrcFile)       ::  rcF
    type(TDate)         ::  tread1, tread2
    type(TDate)         ::  tdyn, tr(2)

    ! --- begin -----------------------------------------------
    
    call goLabel( rname )

    ! set default control parameters
    call default

    if ( myid == root ) then

      !=============================
      ! read parameters from rcfile
      !=============================

      call Init( rcF, rcfile, status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc(rcF, 'time.ndyn_max', ndyn, status )
      IF_NOTOK_RETURN(status=1)
      nconv = ndyn
      nchem = ndyn
      nsrce = ndyn
      ndyn_max = ndyn

      ! ensure that every 'nread' seconds is at the end of a dynamic time step:
      call ReadRc( rcF, 'time.ntimestep', nread, status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc( rcF, 'inputdir', inputdir, status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc( rcF, 'okdebug', okdebug, status )
      IF_NOTOK_RETURN(status=1)

      call Done( rcF, status )
      IF_NOTOK_RETURN(status=1)


      !
      ! read variables from timing rc file
      !

      call Init( rcF, rcfile_runtime, status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc( rcF, 'year1' , idatei(1), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'month1', idatei(2), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'day1'  , idatei(3), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'hour1' , idatei(4), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'minu1' , idatei(5), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'sec1'  , idatei(6), status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc( rcF, 'year2' , idatee(1), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'month2', idatee(2), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'day2'  , idatee(3), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'hour2' , idatee(4), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'minu2' , idatee(5), status )
      IF_NOTOK_RETURN(status=1)
      call ReadRc( rcF, 'sec2'  , idatee(6), status )
      IF_NOTOK_RETURN(status=1)

      idatet = idatee

      call ReadRc( rcF, 'istart', istart, status )
      IF_NOTOK_RETURN(status=1)

      call ReadRc( rcF, 'outputdir', outdir, status )
      IF_NOTOK_RETURN(status=1)

      call Done( rcF, status )
      IF_NOTOK_RETURN(status=1)


      !
      ! checks ...
      !

      !AJS: not necessary anymore ...
      !if (mod(nread,ndyn) /= 0 ) then
      !   print *, 'start: ndyn, nread:', ndyn,nread
      !   call escape_tm('start: mod(nread,ndyn) should be 0')
      !end if

    end if

#ifdef MPI      
    ! broadcast namelist
    call MPI_BCAST(istart   ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ndyn     ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr) 
    call MPI_BCAST(ndyn_max ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)   
    call MPI_BCAST(nconv    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ndiag    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(nchem    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(nsrce    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(nread    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(nwrite   ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ninst    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ndiff    ,1,   MPI_INTEGER,    root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(icalendo ,1,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(iyear0   ,1,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(idatei   ,6,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(idatee   ,6,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(idatet   ,6,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ndiagp1  ,1,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(ndiagp2  ,1,   MPI_INTEGER,     root ,MPI_COMM_WORLD,ierr)  
    call MPI_BCAST(czeta    ,1,   MY_REAL,         root ,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(czetak   ,1,   MY_REAL,         root ,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(limits   ,1,   MPI_LOGICAL,     root ,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(cdebug   ,1,   MPI_LOGICAL,     root ,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(okdebug  ,1,   MPI_LOGICAL,     root ,MPI_COMM_WORLD,ierr)
    !CMK added:
    call MPI_BCAST( inputdir, len(inputdir), MPI_CHARACTER, root, MPI_COMM_WORLD, ierr )
    call MPI_BCAST(   outdir, len(  outdir), MPI_CHARACTER, root, MPI_COMM_WORLD, ierr )
#endif

    !c                             !--------------------------!
    !c                             ! initialize new model run !
    !c                             !--------------------------!
    !c                             ! initialize time parameters
    !c                             ! and set calendar
    !c                             !--------------------------!
    !
    ! itau runs from beginning of year -1 allows running from 1-1-yyyy 
    ! cmk aug/2003
    iyear0 = idatei(1)-1 
    julday0=julday(1,1,iyear0) 
    if (icalendo.eq.2) then
       julday0=julday(1,1,iyear0)
    end if

    call date2tau(idatei,itaui)
    itau=itaui
    call tau2date(itau,idate)
    if(.not.reset) call declare_fields           !allocate memory for this run

    call date2tau(idatee,itaue)
    call date2tau(idatet,itaut)

    ! set time flags
    newyr=.true.
    newday=.true.
    newmonth=.true.
    newsrun=.true.

    if(.not.reset) then

    ! step counter
    nstep0=0
    nstep=0

    ! initialise CFL:
    CALL init_cfl          !initialise CFL preventing routine

    ! cmk: DO THIS FIRST: CHILDREN ARE NEEDED FOR REDGRID TEST!
    call determine_children_etc

    ! set up model geometry...
    if ( grid_reduced .and. myid == root ) then
       do n=1,nregions
          call initredgrid(n,im(n),jm(n))
       end do
    end if

#ifdef MPI      
    call MPI_BCAST( nred     , nredmax         , MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
    call MPI_BCAST( clustsize, nredmax*nregions, MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
    call MPI_BCAST( imred    , 180             , MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
    call MPI_BCAST( imredj   , nredmax*nregions, MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
    call MPI_BCAST( jred     , nredmax*nregions, MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
#endif
    call define_splitorderzoom
    do region = 1,nregions
       call blank_zoom_region(region,region_dat(region))
    end do
    if ( myid == root ) call print_zoom(region_dat)

    !c---- construct horizontal geometry

    ! grid cell area in 1x1 grid
    call calc_dxy(dxy11,nlat180)

    endif ! reset

    ! grid definition
    do region = 1, nregions
       call GeomtryH( region )
    end do
    
    ! setup from start time to end of interval  [k*nread,(k+1)*nread]
    tread1 = NewDate(time6=idate)
    tread2 = NewDate(time6=(/idate(1:3),0,0,0/)) + IncrDate(sec=floor(idate(4)*3600.0/nread)*nread+nread)

    ! n is the number of dynamic intervals within the
    ! time interval for which the meteo has been setup:
    n = ceiling( rTotal(tread2-tread1,'sec') / real(ndyn) )
    ndyn = nint( rTotal(tread2-tread1,'sec') / n )

    ! setup pressure and mass fields
    !  o do not check pressure implied by advection
    call Meteo_Setup_Mass( tread1, tread2, status, isfirst=.true. )
    if (status/=0) then; call goErr; status=1; return; end if

    ! determine dynamic timestep ndyn for this interval [tread1,tread2] ;
    ! the initial number of time steps n is increased until no cfl
    ! violations occure
    call Check_CFL( tread1, tread2, n, status )
    if (status/=0) then; call goErr; status=1; return; end if

    ! * setup meteo for dynamic step tdyn+[0,ndyn]
    ! current time (begin of dynamics step)
    tdyn = NewDate( time6=idate  )
    ! time range of dynamic step:
    tr(1) = tdyn
    tr(2) = tdyn + IncrDate( sec=ndyn )
    !! convert pu/pv to am/bm/cm, eventually time interpolated
    !call Setup_MassFlow( tr, status )
    !if (status/=0) call escape_tm('ERROR in tracer') 
    ! setup (interpolate?) other meteo:
    call Meteo_Setup_Other( tr(1), tr(2), status )
    if (status/=0) then; call goErr; status=1; return; end if


    !
    ! ** init tracer fiels ****************************
    ! 

    ! get initial tracer fields... (m is also available...)
    IF(revert ==1)THEN

      select case (istart)
        case(1)

          !c                             ! initial tracer fields are set
          !c                             ! to zero
          do region = 1,nregions
#ifdef MPI          
            mass_dat(region)%rm_k = 0.0
#endif
            mass_dat(region)%rm_t = 0.0

            if ( adv_scheme == 'slope' .or. adv_scheme == '2nd_m' ) then
#ifdef MPI           
              mass_dat(region)%rxm_k=0.
              mass_dat(region)%rym_k=0.
              mass_dat(region)%rzm_k=0.
#endif
              mass_dat(region)%rxm_t=0.
              mass_dat(region)%rym_t=0.
              mass_dat(region)%rzm_t=0.
            end if
#ifdef secmom
            if ( adv_scheme == '2nd_m' ) then
#ifdef MPI           
               mass_dat(region)%rxxm_k=0.
               mass_dat(region)%rxym_k=0.
               mass_dat(region)%rxzm_k=0.
               mass_dat(region)%ryym_k=0.
               mass_dat(region)%ryzm_k=0.
               mass_dat(region)%rzzm_k=0.
#endif
               mass_dat(region)%rxxm_t=0.
               mass_dat(region)%rxym_t=0.
               mass_dat(region)%rxzm_t=0.
               mass_dat(region)%ryym_t=0.
               mass_dat(region)%ryzm_t=0.
               mass_dat(region)%rzzm_t=0.
            end if
#endif
          end do

        !
        ! initial tracer fields are calculated in trace1
        !

        case ( 2 )

          call trace1

        !
        ! initial tracer fields are obtained
        ! from a save file of a previous
        ! model run. m is also obtained!
        !

        case ( 3 )

          call Init( rcF, rcfile, status )
          IF_NOTOK_RETURN(status=1)

          do region=1,nregions
            call ReadRc( rcF, 'start.hdf.3.'//region_name(region), fname ,status)
            call ReadHDF( region, trim(fname) )
          end do
          call Done( rcF, status )
          IF_NOTOK_RETURN(status=1)

        !
        ! initial tracer fields are obtained
        ! from a save file of a previous
        !

        case(4)

          !mk now HDF
          !WP! read tracer mixing ratios from saveoldfile

          !do region=1,nregions
          !   call readhdfmmr(region,'savetm5.hdf')
          !end do

          call Init( rcF, rcfile, status )
          IF_NOTOK_RETURN(status=1)
          do region=1,nregions
            call ReadRc( rcF, 'start.4.'//region_name(region), fname, status )
            IF_NOTOK_RETURN(status=1)
            call readhdfmmr( region, fname )
          end do
          call Done( rcF, status )
          IF_NOTOK_RETURN(status=1)


        case(5)

          !                             ! initial tracer fields are obtained
          !                             ! from a mmix file.
          !
          ! This is typically the choice for combining different versions
          ! or extending the number of tracers
          ! The compounds are searched by name. If not in the mmix file
          ! the field is initialized as zero

          call Init( rcF, rcfile, status )
          IF_NOTOK_RETURN(status=1)
          do region=1,nregions
            call ReadRc( rcF, 'start.5.'//region_name(region), fname, status )
            IF_NOTOK_RETURN(status=1)
            call readhdfmmix(region,fname)
          end do
          call Done( rcF, status )
          IF_NOTOK_RETURN(status=1)
          
        case(6)   ! start up field from a customized co2 field on 1x1 degree, then coarsened
      
          call start_co2
          
        !
        ! User defined
        !
        
        case ( 9 )
        
          call user_input_start( status )
          IF_NOTOK_RETURN(status=1)
        
        
        !
        ! error ...
        !

        case default

          write (gol,'("unsupported istart : ",i2)') istart; call goErr
          call goErr; status=1; return
          
      end select
      
    ENDIF

    !cmk    transfer information from fine--->coarse...      

    do region = nregions,2,-1
       call update_parent(region)
    end do


    call date2tau(idatee,itaue)
    call date2tau(idatet,itaut)

    if(.not.reset) then
        call cputim(cpu0)
        call Par_Barrier

        call trace0( status )
        if (status/=0) then; call goErr; status=1; return; end if
        call Par_Barrier

        ! initialise user-specified output (!CMK only once!)
        call user_output_init( status )
        if (status/=0) then; call goErr; status=1; return; end if
    
    else

        co2_avg=0.0     ! to make weekly average flux
        n_accumulate=0  ! to make weekly average flux
 
    endif
    if ( myid == root ) then
       call tstamp(kmain,itau,'start: model start up complete')
       write (gol,'(1x)'); call goPr
       write (gol,'(1x,39("--"))'); call goPr
       write (gol,'(1x,39("--"))'); call goPr
       write (gol,'(1x)'); call goPr
    endif
    
    ! ok
    call goLabel(); status=0

  end subroutine start


  !
  ! ***********************************************************************
  ! cpu time
  ! ***********************************************************************
  !

  subroutine cputim(time )
    implicit none

    real,intent(out) :: time
    
    integer :: clock,clockrate

    call system_clock(clock,clockrate)    
    time=clock*(1.0/clockrate)            
    !time=second()        
    
  end subroutine cputim



  ! the following routines are called at start-up to calculate some
  ! useful information for the model, like array adresses and 
  ! the lay-out of the defined regions.


  subroutine determine_children_etc
    !
    ! determines children and ibeg,iend,jbeg,jend,lbeg,lend
    ! the subroutine is normally called once per run of TM5
    ! written by patrick berkvens and mike botchev, march-june 1999
    !
    use dims
    implicit none
    ! local
    integer :: icells, jcells, lcells, jj, region
    integer :: xref_, yref_, zref_

    if ( okdebug ) print *,'determine_children_etc: '

    children = 0
    do region=2,nregions
       ! increase children's counter of the parent:
       children(parent(region),0) = children(parent(region),0) + 1
       jj = children(parent(region),0)
       ! this child is region:
       children(parent(region),jj) = region
    end do
    ! for example:
    !children(0,0) = 1  ! means: there is one child in the region 0
    !children(0,1) = 1  ! means: this child is region 1
    !children(1,0) = 1  ! means: there is one child in region 1
    !children(1,1) = 2  ! means: this child is region 1
    !children(2,0) = 0  ! no children in region 2 

    do region=2,nregions
       ibeg(region) = nint((xbeg(region)-xbeg(parent(region)))    &
            /         (dx/xref(parent(region)))) + 1
       jbeg(region) = nint((ybeg(region)-ybeg(parent(region)))    &
            /         (dy/yref(parent(region)))) + 1
       lbeg(region) = nint((zbeg(region)-zbeg(parent(region)))    &
            /         (dz/zref(parent(region)))) + 1

       iend(region) = nint((xend(region)-xbeg(parent(region)))  &
            /         (dx/xref(parent(region)))) 
       jend(region) = nint((yend(region)-ybeg(parent(region)))  &
            /         (dy/yref(parent(region))))
       lend(region) = nint((zend(region)-zbeg(parent(region)))  &
            /         (dz/zref(parent(region))))

       ! test 
       xref_ = xref(region)/xref(parent(region))
       yref_ = yref(region)/yref(parent(region))
       zref_ = zref(region)/zref(parent(region))

       ! compute icells - how many cells of parent in x-direction a region occupies
       if (iend(region)>ibeg(region)) then
          icells = iend(region)-ibeg(region)+1
       else
          icells = im(parent(region)) - ibeg(region) + 1 + iend(region)
       end if
       if (icells*xref_/=im(region)) then
          print *, 'determine_children_etc: ', &
               'check xbeg/xend, im and ibeg/iend for region ',region
          print *, 'determine_children_etc: xbeg,xend,im,ibeg,iend:', &
               xbeg(region),xend(region),im(region),ibeg(region),iend(region)
          stop
       end if
       jcells = jend(region)-jbeg(region)+1
       if (jcells*yref_/=jm(region)) then
          print *, 'determine_children_etc: ', &
               'check ybeg/yend, jm and jbeg/jend for region ',region
          print *, 'determine_children_etc: ybeg,yend,jm,jbeg,jend:', &
               ybeg(region),yend(region),jm(region),jbeg(region),jend(region)
          stop
       end if
       lcells = lend(region)-lbeg(region)+1
       if (lcells*zref_/=lm(region)) then
          print *, 'determine_children_etc: ', &
               'check zbeg/zend, lm and lbeg/lend for region ',region
          print *, 'determine_children_etc: zbeg,zend,lm,lbeg,lend:', &
               zbeg(region),zend(region),lm(region),lbeg(region),lend(region)
          
       end if
       
    end do
    if ( okdebug ) then
       print *,'determine_children_etc: ibeg(2:nregions): ',ibeg
       print *,'determine_children_etc: iend(2:nregions): ',iend
       print *,'determine_children_etc: jbeg(2:nregions): ',jbeg
       print *,'determine_children_etc: jend(2:nregions): ',jend
       print *,'determine_children_etc: lbeg(2:nregions): ',lbeg
       print *,'determine_children_etc: lend(2:nregions): ',lend
       print *,'determine_children_etc: parents: ',parent
       do region=1,nregions
          print *,'determine_children_etc: children(',region, &
               ',0:nregions)=',children(region,:)
       end do
    end if
    ! determine isr,ier,jsr,jer: the scope of the cells to be 
    ! processed per region.
    ! processes like vertical mixing, chemistry, sources/sinks 
    ! are processed on this scope:
    ! do i=isr(region),ier(region)
    ! do j=jsr(region),jer(region)

    isr(1) = 1  
    ier(1) = im(1)
    jsr(1) = 1
    jer(1) = jm(1)
    do region = 2, nregions
       xref_ = xref(region)/xref(parent(region))
       yref_ = yref(region)/yref(parent(region))
       isr(region) = xref_ + 1
       ier(region) = im(region)-xref_
       jsr(region) = yref_+1
       jer(region) = jm(region)-yref_
       if(xcyc(region)/=0) then
          isr(region) = 1
          ier(region) = im(region)
       end if
       if(touch_sp(region)==1) jsr(region) = 1
       if(touch_np(region)==1) jer(region) = jm(region)
    end do

  end subroutine determine_children_etc



  subroutine define_splitorderzoom
    !
    ! splitorderzoom is splitorder specified for each region 
    ! the subroutine is normally called once per run of TM5
    ! ATTN: subroutine was written based on assumption that a region can not
    ! have number less than number of its parent: region > parent(region)
    ! written by patrick berkvens and mike botchev, march-june 1999
    !
    use dims
    implicit none
    ! local
    integer :: i, j, j0, region, tref_
    !character,dimension(3):: reverse

    if ( okdebug ) then
      write (gol,*) 'define_splitorderzoom: '; call goPr
    end if
    splitorderzoom = ' '
    splitorderzoom(1,1:nsplitsteps) = splitorder
    if ( zoom_mode == 1 ) then
       ! zoom_mode==1 means:
       !  x y z       | z y x
       !  x y z z y x | z y x x y z
       !  ...
       do region=2,nregions
          tref_ = tref(region)/tref(parent(region))
          if ( (tref_ > 4 ) .or. ( tref_ == 3 ) ) then
             write (gol,*) 'define_splitorderzoom: ERROR region = ',region; call goErr
             write (gol,*) 'define_splitorderzoom: ', 'wrong value for tref(region): ',tref(region); call goErr
             write (gol,*) 'define_splitorderzoom: ', 'should be 1,2 or 4 times bigger than tref of its parent'; call goErr
             write (gol,*) 'define_splitorderzoom: ', 'use another value for parameter zoom_mode'; call goErr
             stop
          end if
          j0 = 1  ! step counter for parent
          j = 1   ! step counter for region
          do while ( splitorderzoom(parent(region),j0) /= ' ' )
             ! use step triple of parent tref_ times:
             do i=1,tref_
                if (mod(i,2)==1) then
                   ! copy step triple of parent:
                   !WP! changed 2 into n_oper
                   splitorderzoom(region,j:j+(n_operators-1)) = &
                        splitorderzoom(parent(region),j0:j0+(n_operators-1))
                else
                   ! step triple of parent in reverse order:
                   !    WP! changed 2 into n_op 
                   splitorderzoom(region,j:j+(n_operators-1)) = & 
                        reverse(splitorderzoom(parent(region), &
                        j0:j0+(n_operators-1)))
                end if
                !WP! changed 3 into n_ope...
                j = j + (n_operators)
             end do
             ! next step triple of parent  !WP! changed 3 into n_oper..
             j0 = j0 + (n_operators)
          end do
       end do

    else if ( zoom_mode == 2 ) then

       ! zoom_mode==2 means:
       !  x y z       | z y x
       !  x y z x y z | z y x z y x
       !  ...
       do region=2,nregions
          tref_ = tref(region)/tref(parent(region))
          j0 = 1  ! step counter for parent
          j = 1   ! step counter for region
          do while (splitorderzoom(parent(region),j0)/=' ')
             ! replicate step triple of parent tref_ times:
             do i=1,tref_
                splitorderzoom(region,j:j+(n_operators-1)) = &
                     splitorderzoom(parent(region),j0:j0+(n_operators-1))      
                j = j + (n_operators)
             end do
             ! next step  of parent
             j0 = j0 + (n_operators)
          end do
       end do
    else
       write (gol,*) 'define_splitorderzoom: wrong value for zoom_mode ',zoom_mode; call goErr
       stop
    end if

  contains

    function reverse(str) result(inv_str)
      !WP! changed to n_operators....
      character,dimension(n_operators):: str,inv_str
      integer ::i

      do i=1,(n_operators)
         inv_str(i)=str(n_operators+1-i)
      end do

    end function reverse

  end subroutine define_splitorderzoom


  ! ***
  

  subroutine print_zoom(region_dat)

    use dims

    ! --- in/out ----------------------------------
    
    type(region_data), intent(in)   :: region_dat(nregions)

    ! --- local ---------------------------------
    
    integer :: region, j
    integer ::  n

    ! --- begin ---------------------------------

    write (gol,'("print_zoom")'); call goPr
    write (gol,'("print_zoom: nregions : ",i6)') nregions; call goPr
    write (gol,'("print_zoom: xbeg     : ",20i6)') xbeg; call goPr
    write (gol,'("print_zoom: xend     : ",20i6)') xend; call goPr
    write (gol,'("print_zoom: ybeg     : ",20i6)') ybeg; call goPr
    write (gol,'("print_zoom: yend     : ",20i6)') yend; call goPr
    write (gol,'("print_zoom: zbeg     : ",20i6)') zbeg; call goPr
    write (gol,'("print_zoom: zend     : ",20i6)') zend; call goPr
    write (gol,'("print_zoom: xref     : ",20i6)') xref; call goPr
    write (gol,'("print_zoom: yref     : ",20i6)') yref; call goPr
    write (gol,'("print_zoom: zref     : ",20i6)') zref; call goPr
    write (gol,'("print_zoom: im       : ",20i6)') im; call goPr
    write (gol,'("print_zoom: jm       : ",20i6)') jm; call goPr
    write (gol,'("print_zoom: lm       : ",20i6)') lm; call goPr
    write (gol,'("print_zoom: isr      : ",20i6)') isr; call goPr
    write (gol,'("print_zoom: ier      : ",20i6)') ier; call goPr
    write (gol,'("print_zoom: jsr      : ",20i6)') jsr; call goPr
    write (gol,'("print_zoom: jer      : ",20i6)') jer; call goPr

    write (gol,'("print_zoom: ibeg(2:nregions) : ",20i6)') ibeg; call goPr
    write (gol,'("print_zoom: iend(2:nregions) : ",20i6)') iend; call goPr
    write (gol,'("print_zoom: jbeg(2:nregions) : ",20i6)') jbeg; call goPr
    write (gol,'("print_zoom: jend(2:nregions) : ",20i6)') jend; call goPr
    write (gol,'("print_zoom: lbeg(2:nregions) : ",20i6)') lbeg; call goPr
    write (gol,'("print_zoom: lend(2:nregions) : ",20i6)') lend; call goPr

    write (gol,'("print_zoom: parents : ",20i6)') parent; call goPr
    do region=1,nregions
       write (gol,'("print_zoom: children(",i2,",0:nregions)=",20i3)') region, children(region,:); call goPr
    end do

    do region=1,nregions
       n = count(splitorderzoom(region,:)/=' ')
       write (gol,'("print_zoom: splitorderzoom(",i2,",:)=",100(a1))') &
            region, splitorderzoom(region,1:n); call goPr
    end do

    do region = 1,nregions
       write (gol,'("print_zoom: Zoomed ... region ",i3)') region ; call goPr
       do j=jm(region),1,-1
          write (gol,'("print_zoom: ",i3,a3,200i1)') j,'---',region_dat(region)%zoomed(:,j); call goPr
       end do
       write (gol,'("print_zoom: Edge   ... region ",i3)') region ; call goPr
       do j=jm(region),1,-1
          write (gol,'("print_zoom: ",i3,a3,200i1)') j,'---',region_dat(region)%edge(:,j); call goPr
       end do
    end do

  end subroutine print_zoom

 
  ! ***
  

  subroutine blank_zoom_region(region,region_dat)
    !
    ! determine which cells in a region are calculated in an overlying
    ! zoom region. Only the core of a zoom region is flagged. The
    ! interface cells are not flagged, since they are treated coarse
    ! ref: Krol et al, IAMAS paper, 2001
    ! 
    ! Example    region1:
    ! Zoomed  I2 represent the core-zoom region
    ! 1111111111111111111
    ! 1111111111111111111
    ! 1111222111111111111
    ! 1111222111111111111
    ! 1111222111111111111
    ! 1111222111111111111
    ! 1111111111111111111
    ! 1111111111111111111
    ! Edge     !indicates the interface cells
    ! 0000000000000000000
    ! 0003222300000000000
    ! 0001000100000000000
    ! 0001000100000000000
    ! 0001000100000000000
    ! 0001000100000000000
    ! 0003222300000000000
    ! 0000000000000000000
    !     3 means x+y, 2 means y, and 1 means only x interface cell.
    !
    use dims
    implicit none

    ! input/output
    integer, intent(in)               :: region
    type(region_data),intent(inout)   :: region_dat

    ! local
    integer                           :: i, j, n, n1, xref_, yref_
    integer,dimension(:,:),pointer    :: zoomed, edge

    zoomed => region_dat%zoomed
    edge   => region_dat%edge

    zoomed = region
    edge = 0
    do n=1,children(region,0)  !loop over the children
       n1 = children(region,n)    !child region

       do i=1,im(region)
          if( (i <= ibeg(n1) .or. i >= iend(n1)) .and. (xcyc(n1) /= 1) ) cycle
          do j=1,jm(region)
             if( (j <= jbeg(n1)) .and. (touch_sp(n1)/=1) ) cycle
             if( (j >= jend(n1)) .and. (touch_np(n1)/=1) ) cycle
             zoomed(i,j) = n1
          end do
       end do

       if ( xcyc(n1) /= 1 ) then
          edge(ibeg(n1),jbeg(n1):jend(n1)) = 1
          edge(iend(n1),jbeg(n1):jend(n1)) = 1
       end if
       if ( touch_sp(n1) /= 1 ) then
          edge(ibeg(n1):iend(n1),jbeg(n1)) = &
               edge(ibeg(n1):iend(n1),jbeg(n1)) + 2
       end if
       if ( touch_np(n1) /= 1 ) then
          edge(ibeg(n1):iend(n1),jend(n1)) = &
               edge(ibeg(n1):iend(n1),jend(n1)) + 2
       end if

    end do

    ! put the edge of the region itself to -1.....
    if ( parent(region) /= 0 ) then
       xref_ = xref(region)/xref(parent(region))
       yref_ = yref(region)/yref(parent(region))
       if(xcyc(region)/=1) then
          edge(1:xref_,:) = -1
          edge(im(region)-xref_+1:im(region),:) = -1
       end if
       if(touch_sp(region)/=1) edge(:,1:yref_) = -1
       if(touch_np(region)/=1) edge(:,jm(region)-yref_+1:jm(region)) = -1
    end if

    nullify(zoomed)
    nullify(edge)

  end subroutine blank_zoom_region


end module initexit
