!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module user_output_column

  use dims,       only: nregions
  use GO, only : gol, goErr


  IMPLICIT NONE

  PRIVATE

  !public routines

  PUBLIC :: init_columndata
  PUBLIC :: free_columndata
  PUBLIC :: output_columndata
  PUBLIC :: write_columndata
  PUBLIC :: evaluate_columndata
  PUBLIC :: reset_columndata_accumulator


  !public types
  PUBLIC :: type_column_data

  !public variables
  PUBLIC :: col_data


  CHARACTER(len=200) :: outdir_column


  TYPE type_column_data

    REAL, DIMENSION(:,:), POINTER    :: m_col_mean       ! column value m  - mean
    REAL, DIMENSION(:,:), POINTER    :: ps_col_mean      ! column value ps  - mean
    REAL, DIMENSION(:,:), POINTER    :: m_col_std        ! column value m  - standard deviation
    REAL, DIMENSION(:,:,:), POINTER  :: c_col_mean       ! column value rm/m - mean
    REAL, DIMENSION(:,:,:), POINTER  :: c_col_std        ! column value rm/m - standard deviation
    INTEGER,DIMENSION(:,:), POINTER  :: n_columndata     ! counter (number of accumulations per region)

  END TYPE type_column_data

  TYPE(type_column_data), DIMENSION(nregions) :: col_data


contains

!===========================================================================================================
!===========================================================================================================

SUBROUTINE init_columndata

  use dims,        only: im,jm,lm
  use chem_param,  only: ntrace
  use dims,        only: nregions

  
  IMPLICIT NONE

  !__IO___________________________________________________________________

  !__LOCAL_VARIABLES______________________________________________________

  INTEGER            :: region
  INTEGER            :: imr,jmr,lmr
!  CHARACTER(len=200) :: outdir
  INTEGER            :: slen


  !__START_SUBROUTINE______________________________________________________


  DO region=1,nregions

    imr = im(region)
    jmr = jm(region)
    lmr = lm(region)

    ALLOCATE(col_data(region)%n_columndata(imr,jmr)) 
    ALLOCATE(col_data(region)%m_col_mean(imr,jmr)) 
    ALLOCATE(col_data(region)%ps_col_mean(imr,jmr)) 
    ALLOCATE(col_data(region)%c_col_mean(imr,jmr, 1)) 

    col_data(region)%m_col_mean(:,:) = 0.0
    col_data(region)%ps_col_mean(:,:) = 0.0
    col_data(region)%c_col_mean(:,:,:) = 0.0

    col_data(region)%n_columndata(:,:) = 0

  ENDDO   

END SUBROUTINE init_columndata

!===========================================================================================================
!===========================================================================================================

SUBROUTINE free_columndata

  use dims,       only: nregions

  IMPLICIT NONE

  !__IO___________________________________________________________________

  !__LOCAL_VARIABLES______________________________________________________

  INTEGER   :: region

  !__START_SUBROUTINE______________________________________________________

  DO region=1,nregions
    DEALLOCATE(col_data(region)%m_col_mean) 
    DEALLOCATE(col_data(region)%ps_col_mean) 
    DEALLOCATE(col_data(region)%c_col_mean) 
    DEALLOCATE(col_data(region)%n_columndata) 
  ENDDO   

END SUBROUTINE free_columndata


!===========================================================================================================
!===========================================================================================================

SUBROUTINE output_columndata(region)


  IMPLICIT NONE

  !__IO___________________________________________________________________

  INTEGER                     :: region

  !__LOCAL_VARIABLES______________________________________________________


  !__START_SUBROUTINE______________________________________________________


  CALL accumulate_columndata2(region)



END SUBROUTINE output_columndata
!===========================================================================================================
!===========================================================================================================

SUBROUTINE accumulate_columndata2(region)

  use chem_param,   only: ra, xmair, fscale
  use chem_param,   only: ntrace
  use global_data,  only: mass_dat, region_dat
  use dims,         only: isr,ier,jsr,jer,lm
  use dims,         only: itaur
  use dims,         only: xbeg
  use dims,         only: xref
  use dims,         only: dx
  use binas,        only: Avog
  use datetime,     only: tau2date


  IMPLICIT NONE

  !__IO___________________________________________________________________

  INTEGER                     :: region

  !__LOCAL_VARIABLES______________________________________________________


  REAL, DIMENSION(:,:,:), POINTER      :: m
  REAL, DIMENSION(:,:,:,:), POINTER    :: rm
  INTEGER, DIMENSION(:,:), POINTER     :: zoomed
  REAL,DIMENSION(:)    ,POINTER        :: dxyp
  INTEGER                              :: i,j,l,n
  REAL                                 :: m_col          ! molec / cm2
  REAL, DIMENSION(1)              :: rm_col         ! molec / cm2
  REAL, DIMENSION(1)              :: c_col          ! ppb

  real,dimension(:,:), pointer         :: ps             ! surface pressure

  REAL, DIMENSION(1)              :: scal_c

  REAL                                 :: utc                ! UTC
  REAL                                 :: loct               ! local time (corresponding to exact longitude)
  REAL, parameter                      :: loct_min=10.0-0.5  ! minimum local time for which model output should be sampled
  REAL, parameter                      :: loct_max=10.0+0.5  ! maximum local time for which model output should be sampled

  integer,dimension(6)                 :: idater
  real                                 :: lon
  real                                 :: dxx

  !__START_SUBROUTINE______________________________________________________


  m => mass_dat(region)%m_t
  rm => mass_dat(region)%rm_t
  zoomed => region_dat(region)%zoomed
  dxyp => region_dat(region)%dxyp

  ps => mass_dat(region)%phlb_t(:,:,1)

  ! scaling factor
  !DO n=1, ntrace
  DO n=1, 1
  	scal_c(n) =  fscale(n)
  ENDDO


  call tau2date(itaur(region),idater)
  dxx=dx/xref(region)

  DO i=isr(region),ier(region)

    ! UTC in hours...
    utc = idater(4) + idater(5)/60.0 + idater(6)/3600.0

    ! longitude
    lon=xbeg(region)+(i-0.5)*dxx    !WP! center of current longitude

	! local time
	loct = utc + lon/15.0

    IF (loct < 0.0) loct = loct + 24.0
    IF (loct > 24.0) loct = loct - 24.0


    IF (loct > loct_min .AND. loct < loct_max) THEN
    ! sample only for time interval close to SCIAMACHY overpass time (10 LT)

      DO j=jsr(region),jer(region)

        IF(zoomed(i,j)/=region) CYCLE

        !=============================
        ! accumulate surface pressure
        !=============================

        col_data(region)%ps_col_mean(i,j) =  &
          col_data(region)%ps_col_mean(i,j) + ps(i,j)

        !==========================
        ! accumulate mixing ratio
        !==========================

        DO l=1,lm(region)

          col_data(region)%m_col_mean(i,j) =  &
            col_data(region)%m_col_mean(i,j) + m(i,j,l)

          !DO n=1, ntrace
          DO n=1, 1

            col_data(region)%c_col_mean(i,j,n) =  &
              col_data(region)%c_col_mean(i,j,n) + rm(i,j,l,n) / m(i,j,l) * scal_c(n)

          ENDDO

        ENDDO

        ! increment counter
        col_data(region)%n_columndata(i,j) = col_data(region)%n_columndata(i,j) + 1

      ENDDO

    ENDIF
  ENDDO



END SUBROUTINE accumulate_columndata2

!===========================================================================================================
!===========================================================================================================


!===========================================================================================================
!===========================================================================================================


SUBROUTINE evaluate_columndata(region)

  !==================================================================
  ! evaluate column mean value and std
  !==================================================================

  use global_data,  only: region_dat
  use dims,         only: nregions
  use dims,         only: isr,ier,jsr,jer,lm
  use chem_param,   only: ntrace


  IMPLICIT NONE

  !__IO___________________________________________________________________

  INTEGER                              :: region

  !__LOCAL_VARIABLES______________________________________________________


  INTEGER, DIMENSION(:,:), POINTER     :: zoomed
  INTEGER                              :: i,j,l
  INTEGER                              :: n
  REAL                                 :: temp

  !__START_SUBROUTINE______________________________________________________

  

  !==========================
  ! evaluate mean values
  !==========================  
  

  zoomed => region_dat(region)%zoomed

  DO j=jsr(region),jer(region)
    DO i=isr(region),ier(region)

      IF(zoomed(i,j)/=region) CYCLE

      IF (col_data(region)%n_columndata(i,j) == 0) CYCLE

      col_data(region)%m_col_mean(i,j) =  &
        col_data(region)%m_col_mean(i,j) / col_data(region)%n_columndata(i,j)/lm(1) !WP! mean over time and levels

      col_data(region)%ps_col_mean(i,j) =  &
        col_data(region)%ps_col_mean(i,j) / col_data(region)%n_columndata(i,j) !WP! mean over levels

      !DO n=1, ntrace
      DO n=1, 1

        col_data(region)%c_col_mean(i,j,n) =  &
          col_data(region)%c_col_mean(i,j,n) / col_data(region)%n_columndata(i,j)/lm(1) !WP! mean over time and levels
      ENDDO    

    ENDDO
  ENDDO

  CALL update_parent_columns(region)

END SUBROUTINE evaluate_columndata

!=====================================================================================================
!=====================================================================================================

SUBROUTINE update_parent_columns(region)

  use dims
  use global_data,  only: region_dat
  use toolbox,      only : escape_tm
  use chem_param,   only: ntrace

  IMPLICIT NONE

  !__IO___________________________________________________________________

  INTEGER, INTENT(IN)   :: region

  !__LOCAL_VARIABLES______________________________________________________

  REAL, DIMENSION(:,:), POINTER    :: m_col_mean       ! column value m  - mean
  REAL, DIMENSION(:,:), POINTER    :: ps_col_mean      ! column value ps  - mean
  REAL, DIMENSION(:,:), POINTER    :: m_col_std        ! column value m  - standard deviation
  REAL, DIMENSION(:,:,:), POINTER  :: c_col_mean       ! column value rm/m - mean
  REAL, DIMENSION(:,:,:), POINTER  :: c_col_std        ! column value rm/m - standard deviation

  REAL, DIMENSION(:,:), POINTER    :: p_m_col_mean     ! column value m  - mean
  REAL, DIMENSION(:,:), POINTER    :: p_ps_col_mean    ! column value ps  - mean
  REAL, DIMENSION(:,:), POINTER    :: p_m_col_std      ! column value m  - standard deviation
  REAL, DIMENSION(:,:,:), POINTER  :: p_c_col_mean     ! column value rm/m - mean
  REAL, DIMENSION(:,:,:), POINTER  :: p_c_col_std      ! column value rm/m - standard deviation

  REAL, DIMENSION(:), POINTER    :: dxyp

  INTEGER :: i,j
  INTEGER :: ip, jp
  INTEGER :: my_parent
  INTEGER :: xref_,yref_,zref_

  INTEGER :: imp,jmp
  INTEGER :: imr,jmr

  INTEGER :: ic,jc

  INTEGER :: iox,ioy1,ioy2
  REAL    :: w,wtot



  !__START_SUBROUTINE______________________________________________________



  IF (region==1) RETURN


  imr = im(region)
  jmr = jm(region)

  ! determine parent
  my_parent = parent(region)

  xref_ = xref(region)/xref(my_parent)
  yref_ = yref(region)/yref(my_parent)

  imp = im(region)/xref_
  jmp = jm(region)/yref_

  ! check calculated imp, jmp, lmp
  IF(ibeg(region) < iend(region) .and. imp /= iend(region)-ibeg(region)+1) THEN
    call escape_tm('stopped in update_parent_columns')
  ENDIF
    
  IF(jmp.ne.jend(region)-jbeg(region)+1) THEN
    call escape_tm('stopped in update_parent_columns')
  ENDIF  
 

  m_col_mean  => col_data(region)%m_col_mean
  ps_col_mean => col_data(region)%ps_col_mean
  c_col_mean  => col_data(region)%c_col_mean

  p_m_col_mean  => col_data(my_parent)%m_col_mean
  p_ps_col_mean => col_data(my_parent)%ps_col_mean
  p_c_col_mean  => col_data(my_parent)%c_col_mean


  dxyp=> region_dat(region)%dxyp


  iox = isr(region)/xref_
  ioy1 = jsr(region)/yref_
  ioy2 = (jm(region)-jer(region)+1)/yref_


  DO jp=jbeg(region)+ioy1, jend(region)-ioy2  
    jc = (jp-jbeg(region))*yref_
    DO ip=ibeg(region)+iox, iend(region)-iox
      ic = (ip-ibeg(region))*xref_

      p_m_col_mean(ip,jp) = 0.0
      p_ps_col_mean(ip,jp) = 0.0
      p_c_col_mean(ip,jp,1:1) = 0.0


      wtot = 0.0
      DO j=1,yref_
        w = dxyp(jc+j)
        DO i=1,xref_

          p_m_col_mean(ip,jp) = p_m_col_mean(ip,jp)  + m_col_mean(ic+i,jc+j)*w
          p_ps_col_mean(ip,jp) = p_ps_col_mean(ip,jp)  + ps_col_mean(ic+i,jc+j)*w
          p_c_col_mean(ip,jp,1:1) = p_c_col_mean(ip,jp,1:1)  + c_col_mean(ic+i,jc+j,1:1)*w

          wtot = wtot+w
        ENDDO
      ENDDO
      p_m_col_mean(ip,jp) = p_m_col_mean(ip,jp)/wtot
      p_ps_col_mean(ip,jp) = p_ps_col_mean(ip,jp)/wtot
      p_c_col_mean(ip,jp,1:1) = p_c_col_mean(ip,jp,1:1)/wtot


    ENDDO
  ENDDO


  NULLIFY(m_col_mean)  
  NULLIFY(ps_col_mean)  
  NULLIFY(c_col_mean)  

  NULLIFY(p_m_col_mean)  
  NULLIFY(p_ps_col_mean)  
  NULLIFY(p_c_col_mean)  

END SUBROUTINE update_parent_columns

!===========================================================================================================
!===========================================================================================================

SUBROUTINE write_columndata(region,status)


  use dims
  use chem_param
  use io_hdf
  use datetime,        only : tau2date
  use global_data,     only : outdir
  use file_hdf       , only : THdfFile, Init, Done, WriteAttribute
  use ParTools       , only : myid, root_t

  IMPLICIT NONE

  !__IO___________________________________________________________________
  integer, intent(out)          ::  status

  ! --- const ----------------------------------
    
  character(len=*), parameter  ::  rname = mname//'/WriteColumn'

  !__LOCAL_VARIABLES______________________________________________________

  type(THdfFile)            ::  hdf

  INTEGER                   :: idate4DVAR(6)
  INTEGER                   :: slen        ! string length
  CHARACTER(len = 200)      :: FFilename   ! 
  CHARACTER(len = 5)        :: runid
  INTEGER                   :: region
  INTEGER, DIMENSION(6)     :: idate_f
  INTEGER                   :: n

  !hdf
  INTEGER                    :: sfstart
  INTEGER                    :: sfsnatt
  INTEGER                    :: sfscatt
  INTEGER                    :: io
  INTEGER                    :: istat




  !__START_SUBROUTINE______________________________________________________

    status=0
    if (myid.ne.root_t) return  ! only one pe does I/O

    call tau2date(itaur(region),idate_f)

    WRITE(FFilename,'(a,i4.4,2(i2.2),a1,a6,a4)') &
        trim(outdir)//'column_', idate_f(1:3), '_', region_name(region), '.hdf'

    call Init( hdf, trim(Ffilename), 'create', status )
    IF_ERROR_RETURN(status=1)

    call WriteAttribute( hdf, 'itau'       , itau               , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nregions'   , nregions           , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'region_name', region_name(region), status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'im'         , im(region)         , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'jm'         , jm(region)         , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'lm'         , lm(region)         , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'dx'         , dx/xref(region)    , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'dy'         , dy/yref(region)    , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'dz'         , dz/zref(region)    , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'xbeg'       , xbeg(region)       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'xend'       , xend(region)       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ybeg'       , ybeg(region)       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'yend'       , yend(region)       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'zbeg'       , zbeg(region)       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'zend'       , zend(region)       , status )
    IF_ERROR_RETURN(status=1)

    if ( region /= 1 ) then
      call WriteAttribute( hdf, 'ibeg', ibeg(region), status )
      IF_ERROR_RETURN(status=1)
      call WriteAttribute( hdf, 'iend', iend(region), status )
      IF_ERROR_RETURN(status=1)
      call WriteAttribute( hdf, 'jbeg', jbeg(region), status )
      IF_ERROR_RETURN(status=1)
      call WriteAttribute( hdf, 'jend', jend(region), status )
      IF_ERROR_RETURN(status=1)
      call WriteAttribute( hdf, 'lbeg', lbeg(region), status )
      IF_ERROR_RETURN(status=1)
      call WriteAttribute( hdf, 'lend', lend(region), status )
      IF_ERROR_RETURN(status=1)
    end if

    call WriteAttribute( hdf, 'xref'       , xref(region), status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'yref'       , yref(region), status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'zref'       , zref(region), status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'tref'       , tref(region), status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ntracet'    , ntracet     , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ntrace '    , ntrace      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nstd'       , nstd        , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'idate'      , idate       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'istart'     , istart      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ndyn'       , ndyn        , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nconv'      , nconv       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ndiag'      , ndiag       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nchem'      , nchem       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nsrce'      , nsrce       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nread'      , nread       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nwrite'     , nwrite      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ninst'      , ninst       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ncheck'     , ncheck      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ndiff'      , ndiff       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'itaui'      , itaui       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'itaue'      , itaue       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'itaut'      , itaut       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'itau0'      , itau0       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'idatei'     , idatei      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'idatee'     , idatee      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'idatet'     , idatet      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'idate0'     , idate0      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'icalendo'   , icalendo    , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'iyear0'     , iyear0      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'julday0'    , julday0     , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ndiagp1'    , ndiagp1     , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ndiagp2'    , ndiagp2     , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nstep'      , nstep       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'cpu0'       , cpu0        , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'cpu1'       , cpu1        , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'ra'         , ra          , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'fscale'     , fscale      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'names'      , names       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'areag'      , areag       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'czeta'      , czeta       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'czetak'     , czetak      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'xlabel'     , xlabel      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'istd'       , istd        , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'newyr'      , newyr       , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'newmonth'   , newmonth    , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'newday'     , newday      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'newsrun'    , newsrun     , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'cdebug'     , cdebug      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'limits'     , limits      , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'at'         , at          , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'bt'         , bt          , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'adv_scheme' , adv_scheme  , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'nsplitsteps', nsplitsteps , status )
    IF_ERROR_RETURN(status=1)
    call WriteAttribute( hdf, 'splitorder' , splitorder  , status )
    IF_ERROR_RETURN(status=1)

    CALL io_write(hdf%id, im(region),'i',jm(region),'j', &
          col_data(region)%m_col_mean(1:im(region),1:jm(region)), 'm_col_mean', idate_f)
    CALL io_write(hdf%id, im(region),'i',jm(region),'j', &
          col_data(region)%ps_col_mean(1:im(region),1:jm(region)), 'ps_col_mean', idate_f)
    CALL io_write(hdf%id, im(region),'i',jm(region),'j', &
          col_data(region)%n_columndata(1:im(region),1:jm(region)), 'n_columndata', idate_f)

  !DO n=1,ntrace
  DO n=1,1
!   name=names(n)
    call io_write(hdf%id, & 
        im(region),'i', &
        jm(region),'j', & 
        col_data(region)%c_col_mean(1:im(region),1:jm(region),n), 'c_col_mean', idate_f)
         
  ENDDO

  call Done( hdf, status )
  IF_ERROR_RETURN(status=1)
  status=0

END SUBROUTINE write_columndata

!===========================================================================================================
!===========================================================================================================


SUBROUTINE reset_columndata_accumulator(region)

!==================================================================
! initialize accumulation arrays for average station concentration
!==================================================================

  use dims,         only: nregions

  IMPLICIT NONE

  !__IO___________________________________________________________________

  INTEGER    :: region

  !__LOCAL_VARIABLES______________________________________________________

  !__START_SUBROUTINE______________________________________________________


  col_data(region)%m_col_mean(:,:) = 0.0
  col_data(region)%ps_col_mean(:,:) = 0.0
  col_data(region)%c_col_mean(:,:,:) = 0.0
  col_data(region)%n_columndata(:,:) = 0
  
  
END SUBROUTINE reset_columndata_accumulator


!===========================================================================================================
!===========================================================================================================


end module user_output_column
